@@ -59,7 +59,7 @@ class PlugwiseCircle(PlugwiseNode):
             SENSOR_POWER_CONSUMPTION_TODAY["id"],
             SENSOR_POWER_CONSUMPTION_YESTERDAY["id"],
             SENSOR_POWER_PRODUCTION_CURRENT_HOUR["id"],
-            #SENSOR_POWER_PRODUCTION_PREVIOUS_HOUR["id"],
+            # SENSOR_POWER_PRODUCTION_PREVIOUS_HOUR["id"],
             SENSOR_RSSI_IN["id"],
             SENSOR_RSSI_OUT["id"],
         )
@@ -68,14 +68,15 @@ class PlugwiseCircle(PlugwiseNode):
         self.pulses_8s = None
         self.pulses_consumed_1h = None
         self.pulses_produced_1h = None
+        self.calibration = False
         self._gain_a = None
         self._gain_b = None
         self._off_noise = None
         self._off_tot = None
         self.power_history = {}
-        self.power_consumption_prev_hour = 0
-        self.power_consumption_today = 0
-        self.power_consumption_yesterday = 0
+        self.power_consumption_prev_hour = None
+        self.power_consumption_today = None
+        self.power_consumption_yesterday = None
         self._request_calibration()
 
     def _request_calibration(self, callback=None):
@@ -104,13 +105,20 @@ class PlugwiseCircle(PlugwiseNode):
         Process received message
         """
         if isinstance(message, CirclePowerUsageResponse):
-            self._response_power_usage(message)
-            self.stick.logger.debug(
-                "Power update for %s, last update %s",
-                self.get_mac(),
-                str(self.last_update),
-            )
-            self.stick.message_processed(message.seq_id)
+            if self.calibration:
+                self._response_power_usage(message)
+                self.stick.logger.debug(
+                    "Power update for %s, last update %s",
+                    self.get_mac(),
+                    str(self.last_update),
+                )
+                self.stick.message_processed(message.seq_id)
+            else:
+                self.stick.logger.debug(
+                    "Received power update for %s before calibration information is known",
+                    self.get_mac(),
+                )
+                self._request_calibration()
         elif isinstance(message, CircleSwitchResponse):
             self._response_switch(message)
             self.stick.logger.debug(
@@ -123,11 +131,18 @@ class PlugwiseCircle(PlugwiseNode):
             self._response_calibration(message)
             self.stick.message_processed(message.seq_id)
         elif isinstance(message, CirclePowerBufferResponse):
-            self._response_power_buffer(message)
-            self.stick.message_processed(message.seq_id)
+            if self.calibration:
+                self._response_power_buffer(message)
+                self.stick.message_processed(message.seq_id)
+            else:
+                self.stick.logger.debug(
+                    "Received power buffer log for %s before calibration information is known",
+                    self.get_mac(),
+                )
+                self._request_calibration()
         else:
             self._circle_plus_message(message)
-        
+
     def _circle_plus_message(self, message):
         pass
 
@@ -148,40 +163,36 @@ class PlugwiseCircle(PlugwiseNode):
         Returns power usage during the last second in Watts
         Based on last received power usage information 
         """
-        if self.pulses_1s == None:
-            return 0.0
-        return self.pulses_to_kWs(self.pulses_1s) * 1000
+        if self.pulses_1s:
+            return self.pulses_to_kWs(self.pulses_1s) * 1000
+        return None
 
     def get_power_usage_8_sec(self):
         """
         Returns power usage during the last 8 second in Watts
         Based on last received power usage information 
         """
-        if self.pulses_8s == None:
-            return 0.0
-        return self.pulses_to_kWs(self.pulses_8s, 8) * 1000
+        if self.pulses_8s:
+            return self.pulses_to_kWs(self.pulses_8s, 8) * 1000
+        return None
 
     def get_power_consumption_current_hour(self):
         """
         Returns the power usage during this running hour in kWh
         Based on last received power usage information 
         """
-        if self.pulses_consumed_1h == None:
-            return 0.0
-        elif self.pulses_consumed_1h == 0:
-            return 0.0
-        return self.pulses_to_kWs(self.pulses_consumed_1h, 3600)
+        if self.pulses_consumed_1h:
+            return self.pulses_to_kWs(self.pulses_consumed_1h, 3600)
+        return None
 
     def get_power_production_current_hour(self):
         """
         Returns the power production during this running hour in kWh
         Based on last received power usage information 
         """
-        if self.pulses_produced_1h == None:
-            return 0.0
-        elif self.pulses_produced_1h == 0:
-            return 0.0
-        return self.pulses_to_kWs(self.pulses_produced_1h, 3600)
+        if self.pulses_produced_1h:
+            return self.pulses_to_kWs(self.pulses_produced_1h, 3600)
+        return None
 
     def get_power_consumption_prev_hour(self):
         """ Returns power consumption during the previous hour in kWh
@@ -211,64 +222,68 @@ class PlugwiseCircle(PlugwiseNode):
                 self.do_callback(SWITCH_RELAY["id"])
 
     def _response_power_usage(self, message):
-        # sometimes the circle returns max values for some of the pulse counters
-        # I have no idea what it means but it certainly isn't a reasonable value
-        # so I just assume that it's meant to signal some kind of a temporary error condition
+        # Sometimes the circle returns -1 for some of the pulse counters
+        # likely this means the circle measures very little power and is suffering from
+        # rounding errors. Zero these out. However, negative pulse values are valid
+        # for power producing appliances, like solar panels, so don't complain too loudly.
 
         # Power consumption last second
-        if message.pulse_1s.value == 65535:
+        if message.pulse_1s.value == -1:
+            message.pulse_1s.value = 0
             self.stick.logger.debug(
-                "1 sec power pulse counter for node %s has unreasonable value of 65535",
+                "1 sec power pulse counter for node %s has value of -1, corrected to 0",
                 self.get_mac(),
             )
-        else:
-            self.pulses_1s = message.pulse_1s.value
-            if message.pulse_1s.value != 0:
-                if message.nanosecond_offset.value !=0:
-                    pulses_1s = (message.pulse_1s.value * (1000000000 + message.nanosecond_offset.value)) / 1000000000
-                else:
-                    pulses_1s = message.pulse_1s.value
-                self.pulses_1s = pulses_1s
+        self.pulses_1s = message.pulse_1s.value
+        if message.pulse_1s.value != 0:
+            if message.nanosecond_offset.value != 0:
+                pulses_1s = (
+                    message.pulse_1s.value
+                    * (1000000000 + message.nanosecond_offset.value)
+                ) / 1000000000
             else:
-                self.pulses_1s = 0
-            self.do_callback(SENSOR_POWER_USE["id"])
-
+                pulses_1s = message.pulse_1s.value
+            self.pulses_1s = pulses_1s
+        else:
+            self.pulses_1s = 0
+        self.do_callback(SENSOR_POWER_USE["id"])
         # Power consumption last 8 seconds
-        if message.pulse_8s.value == 65535:
+        if message.pulse_8s.value == -1:
+            message.pulse_8s.value = 0
             self.stick.logger.debug(
-                "8 sec power pulse counter for node %s has unreasonable value of 65535",
+                "8 sec power pulse counter for node %s has value of -1, corrected to 0",
                 self.get_mac(),
             )
-        else:
-            if message.pulse_8s.value != 0:
-                if message.nanosecond_offset.value !=0:
-                    pulses_8s = (message.pulse_8s.value * (1000000000 + message.nanosecond_offset.value)) / 1000000000
-                else:
-                    pulses_8s = message.pulse_8s.value
-                self.pulses_8s = pulses_8s
+        if message.pulse_8s.value != 0:
+            if message.nanosecond_offset.value != 0:
+                pulses_8s = (
+                    message.pulse_8s.value
+                    * (1000000000 + message.nanosecond_offset.value)
+                ) / 1000000000
             else:
-                self.pulses_8s = 0
-            self.do_callback(SENSOR_POWER_USE_LAST_8_SEC["id"])
-
+                pulses_8s = message.pulse_8s.value
+            self.pulses_8s = pulses_8s
+        else:
+            self.pulses_8s = 0
+        self.do_callback(SENSOR_POWER_USE_LAST_8_SEC["id"])
         # Power consumption current hour
-        if message.pulse_hour_consumed.value == 4294967295:
+        if message.pulse_hour_consumed.value == -1:
+            message.pulse_hour_consumed.value = 0
             self.stick.logger.debug(
-                "1 hour consumption power pulse counter for node %s has unreasonable value of 4294967295",
+                "1 hour consumption power pulse counter for node %s has value of -1, corrected to 0",
                 self.get_mac(),
             )
-        else:
-            self.pulses_consumed_1h = message.pulse_hour_consumed.value
-            self.do_callback(SENSOR_POWER_CONSUMPTION_CURRENT_HOUR["id"])
-
+        self.pulses_consumed_1h = message.pulse_hour_consumed.value
+        self.do_callback(SENSOR_POWER_CONSUMPTION_CURRENT_HOUR["id"])
         # Power produced current hour
-        if message.pulse_hour_produced.value == 4294967295:
+        if message.pulse_hour_produced.value == -1:
+            message.pulse_hour_produced.value = 0
             self.stick.logger.debug(
-                "1 hour power production pulse counter for node %s has unreasonable value of 4294967295",
+                "1 hour power production pulse counter for node %s has value of -1, corrected to 0",
                 self.get_mac(),
             )
-        else:
-            self.pulses_produced_1h = message.pulse_hour_produced.value
-            self.do_callback(SENSOR_POWER_PRODUCTION_CURRENT_HOUR["id"])
+        self.pulses_produced_1h = message.pulse_hour_produced.value
+        self.do_callback(SENSOR_POWER_PRODUCTION_CURRENT_HOUR["id"])
 
     def _response_calibration(self, message):
         """ Store calibration properties
@@ -276,15 +291,14 @@ class PlugwiseCircle(PlugwiseNode):
         for x in ("gain_a", "gain_b", "off_noise", "off_tot"):
             val = getattr(message, x).value
             setattr(self, "_" + x, val)
+        self.calibration = True
 
     def pulses_to_kWs(self, pulses, seconds=1):
         """
         converts the amount of pulses to kWs using the calaboration offsets
         """
-        if pulses == 0:
+        if pulses == 0 or not self.calibration:
             return 0.0
-        if self._gain_a is None:
-            return None
         pulses_per_s = pulses / float(seconds)
         corrected_pulses = seconds * (
             (
@@ -293,7 +307,11 @@ class PlugwiseCircle(PlugwiseNode):
             )
             + self._off_tot
         )
-        return corrected_pulses / PULSES_PER_KW_SECOND / seconds
+        calc_value = corrected_pulses / PULSES_PER_KW_SECOND / seconds
+        # Fix minor miscalculations
+        if calc_value < 0.001 and calc_value > -0.001:
+            calc_value = 0.0
+        return calc_value
 
     def _request_power_buffer(self, log_address=None, callback=None):
         """Request power log of specified address
@@ -303,9 +321,7 @@ class PlugwiseCircle(PlugwiseNode):
         if log_address != None:
             if bool(self.power_history):
                 # Only request last 2 power buffer logs
-                self.stick.send(
-                    CirclePowerBufferRequest(self.mac, log_address - 1),
-                )
+                self.stick.send(CirclePowerBufferRequest(self.mac, log_address - 1),)
                 self.stick.send(
                     CirclePowerBufferRequest(self.mac, log_address), callback,
                 )
@@ -319,14 +335,13 @@ class PlugwiseCircle(PlugwiseNode):
                 self.stick.send(
                     CirclePowerBufferRequest(self.mac, log_address), callback,
                 )
-        
+
     def _response_power_buffer(self, message):
         """returns information about historical power usage
         each response contains 4 log buffers and each log buffer contains data for 1 hour
         """
         if message.logaddr.value == self._last_log_address:
             self._last_log_collected = True
-
         # Collect logged power usage
         for i in range(1, 5):
             if getattr(message, "logdate%d" % (i,)).value != None:
@@ -334,32 +349,39 @@ class PlugwiseCircle(PlugwiseNode):
                 if getattr(message, "pulses%d" % (i,)).value == 0:
                     self.power_history[dt] = 0.0
                 else:
-                    self.power_history[dt] = self.pulses_to_kWs(getattr(message, "pulses%d" % (i,)).value, 3600)
-
+                    self.power_history[dt] = self.pulses_to_kWs(
+                        getattr(message, "pulses%d" % (i,)).value, 3600
+                    )
         # Cleanup history for more than 2 day's ago
         if len(self.power_history.keys()) > 48:
             for dt in list(self.power_history.keys()):
-                if (dt + self.stick.timezone_delta - timedelta(hours=1)).date() < (datetime.now().today().date() - timedelta(days=1)):
+                if (dt + self.stick.timezone_delta - timedelta(hours=1)).date() < (
+                    datetime.now().today().date() - timedelta(days=1)
+                ):
                     del self.power_history[dt]
-
-        # Recalculate power use counters        
+        # Recalculate power use counters
         last_hour_usage = 0
         today_power = 0
         yesterday_power = 0
         for dt in self.power_history:
-            if (dt + self.stick.timezone_delta) == datetime.now().today().replace(minute=0, second=0, microsecond=0):
+            if (dt + self.stick.timezone_delta) == datetime.now().today().replace(
+                minute=0, second=0, microsecond=0
+            ):
                 last_hour_usage = self.power_history[dt]
-            if (dt + self.stick.timezone_delta - timedelta(hours=1)).date() == datetime.now().today().date():
+            if (
+                dt + self.stick.timezone_delta - timedelta(hours=1)
+            ).date() == datetime.now().today().date():
                 today_power += self.power_history[dt]
-            if (dt + self.stick.timezone_delta - timedelta(hours=1)).date() == (datetime.now().today().date() - timedelta(days=1)):
+            if (dt + self.stick.timezone_delta - timedelta(hours=1)).date() == (
+                datetime.now().today().date() - timedelta(days=1)
+            ):
                 yesterday_power += self.power_history[dt]
-        if self.power_consumption_prev_hour != round(last_hour_usage, 3):
-            self.power_consumption_prev_hour = round(last_hour_usage, 3)
+        if self.power_consumption_prev_hour != last_hour_usage:
+            self.power_consumption_prev_hour = last_hour_usage
             self.do_callback(SENSOR_POWER_CONSUMPTION_PREVIOUS_HOUR["id"])
-        if self.power_consumption_today != round(today_power, 3):
-            self.power_consumption_today = round(today_power, 3)
+        if self.power_consumption_today != today_power:
+            self.power_consumption_today = today_power
             self.do_callback(SENSOR_POWER_CONSUMPTION_TODAY["id"])
-        if self.power_consumption_yesterday != round(yesterday_power, 3):
-            self.power_consumption_yesterday = round(yesterday_power, 3)
+        if self.power_consumption_yesterday != yesterday_power:
+            self.power_consumption_yesterday = yesterday_power
             self.do_callback(SENSOR_POWER_CONSUMPTION_YESTERDAY["id"])
-
